@@ -3,6 +3,7 @@
 #include <stdlib.h>
 #include <float.h>
 #include <string.h>
+#include <stdio.h>
 
 /*
 =head1 NAME
@@ -1576,11 +1577,11 @@ http://www.exif.org/
 
 =head1 AUTHOR
 
-Tony Cook <tony@imager.perl.org>
+Tony Cook <tonyc@cpan.org>
 
 =head1 REVISION
 
-$Revision: 1814 $
+$Revision$
 
 =cut
 */
