@@ -23,7 +23,7 @@ void
 flines(void *INP) {
   i_img *im;
   i_color vl;
-  int x,y;
+  i_img_dim x,y;
   
   if ( !getOBJ("image","Imager::ImgRaw",&im) ) {
 		fprintf(stderr,"Error: image is missing\n"); 
@@ -31,20 +31,20 @@ flines(void *INP) {
 	}
   
   fprintf(stderr, "flines: parameters: (im %p)\n",im);
-  fprintf(stderr, "flines: image info:\n size (%d,%d)\n channels (%d)\n",
-	  im->xsize,im->ysize,im->channels);
+  fprintf(stderr, "flines: image info:\n size (" i_DFp ")\n channels (%d)\n",
+	  i_DFcp(im->xsize,im->ysize), im->channels);
 
   for(y = 0; y < im->ysize; y ++) {
     for(x = 0; x < im->xsize; x ++ ) {
       i_gpix(im,x,y,&vl); 
 			if (!(y%2)) {
-				float yf = y/(float)im->ysize;
+				float yf = y/(double)im->ysize;
 				float mf = 1.2-0.8*yf;
 				vl.rgb.r = saturate(vl.rgb.r*mf);
 				vl.rgb.g = saturate(vl.rgb.g*mf);
 				vl.rgb.b = saturate(vl.rgb.b*mf);
 			} else {
-				float yf = (im->ysize-y)/(float)im->ysize;
+				float yf = (im->ysize-y)/(double)im->ysize;
 				float mf = 1.2-0.8*yf;
 				vl.rgb.r = saturate(vl.rgb.r*mf);
 				vl.rgb.g = saturate(vl.rgb.g*mf);
